import prompt_toolkit as pt
from prompt_toolkit.shortcuts.prompt import CompleteStyle
from prompt_toolkit.completion import WordCompleter
import questionary


def multi_select(msg: str = "Select an option", choices: list[str] = []):
  value = pt.prompt(
      msg,
      completer=WordCompleter(choices, ignore_case=True),
      complete_while_typing=True,
  )

  return value.strip()


def ask_autocomplete(choices: list[str], question: str = "Select an option", ):

  return questionary.autocomplete(
      question,
      complete_style=CompleteStyle.MULTI_COLUMN,
      choices=choices,
  ).ask()


def ask_boolean(question: str = "Select an option", ):
  return questionary.confirm(
      question,
  ).ask()


def ask_multiselect(choices: list[str], question: str = "Select an option", ):
  return questionary.checkbox(
      question,
      choices=choices,
  ).ask()


if __name__ == "__main__":
  pass

  # print(ask_boolean("Are you happy with the encoding?"))
  #   ask_autocomplete(["a", "b", "c"], "Select an option")



import hashlib
import subprocess
import src.prompts as prompts
import src.file as file


def hash(cmd: str) -> str:
  return hashlib.sha256(cmd.encode("utf-8")).hexdigest()[0:8]


def get_encode_cmd(input_file_path: str, preset: str):
  file_name_without_extension = input_file_path.rsplit(".", 1)[0]
  output_file_path = f"{file_name_without_extension}_HandBraked-{preset}.mp4"
  cmd = f"HandBrakeCLI -i '{input_file_path}' -o '{output_file_path}' -Z '{preset}'"
  file.manager.add_file(output_file_path)

  return cmd


def get_presets():
  cmd = "HandBrakeCLI -z"
  result = subprocess.run(cmd, shell=True, capture_output=True)
  output = result.stderr.decode("utf-8")

  lines = output.splitlines()
  preset_map: dict[str, list[str]] = {}

  category = ""

  for line in lines:
    num_of_space_in_the_beginning = len(line) - len(line.lstrip())
    is_category = "/" in line
    is_preset = num_of_space_in_the_beginning == 4

    if is_category:
      category = line.strip("/")
      preset_map[category] = []

    if len(category) > 0 and is_preset:
      preset_map[category].append(line.strip().strip("\t"))

  return preset_map


def select_preset():
  categories = get_presets()
  category = prompts.ask_autocomplete(choices=list(categories.keys()), question="Select a category")
  presets = categories[category]
  preset = prompts.ask_autocomplete(choices=presets, question="Select a preset")

  return preset


def verify_installation(program_name: str):
  cmd = f"which {program_name}"
  result = subprocess.run(cmd, shell=True, capture_output=True)
  output_text = result.stdout.decode("utf-8")
  stderr_text = result.stderr.decode("utf-8")

  if len(output_text) == 0 or len(stderr_text) > 0:
    raise Exception("HandBrakeCLI is not installed",
                    stderr_text or output_text or "No output")



import src.event as event
import src.loader as loader
import src.utils as utils
from rich import print
from rich.panel import Panel
from rich.syntax import Syntax


def manage(cmds: list[str]):
  loader.loader.start()

  def handle_progress(cmd: str, percent: int):
    loader.loader.update(utils.hash(cmd), percent)

  def get_process(cmd: str):
    return event.ProcessWithEvent(cmd, progress_cb=handle_progress)

  pwe_list = list(map(get_process, cmds))

  for pwe in pwe_list:
    loader.loader.add_task(utils.hash(pwe.cmd), 100)

  for pwe in pwe_list:
    syntax = Syntax(pwe.cmd, "bash", theme="monokai", word_wrap=True)
    panel = Panel(syntax, title="Command", )

    print(panel)
    pwe.start()

  loader.loader.stop()


def main():

  cmd1 = "source /Users/tahsin/.commonrc && hb_test 10"
  cmd2 = "source /Users/tahsin/.commonrc && hb_test 12"
  cmd3 = "source /Users/tahsin/.commonrc && hb_test 5"
  cmd4 = "ls"

  cmds = [cmd1, cmd2, cmd3, cmd4]

  manage(cmds)


if __name__ == "__main__":
  main()



import subprocess
import os
import tempfile
import src.prompts as prompts

output_path = os.path.join(tempfile.gettempdir(), "OUTFILE")
cmd = f"ranger --choosefiles={output_path}"


def select_files() -> list[str]:
  completed_process = subprocess.run(cmd, shell=True)
  if completed_process.returncode != 0:
    raise Exception("Something went wrong while selecting files")

  allowed_extensions = ["mp4", "mkv", "avi", "mov", "m4v", "flv", "wmv"]

  if not os.path.exists(output_path):
    return []

  with open(output_path, "r") as f:
    data = f.read()
    filtered_data = map(lambda x: x.strip(), data.splitlines())
    filtered_data = list(filter(lambda x: len(x) > 0, filtered_data))
    filtered_data = list(filter(lambda x: x.split(
        ".")[-1] in allowed_extensions, filtered_data))

  os.remove(output_path)

  return filtered_data


class _Manager:
  files: list[str] = []

  def add_file(self, file_path: str) -> None:
    """Add a file to the list that has been interacted with"""
    self.files.append(file_path)

  def delete_file(self):
    """Delete files from the list that has been interacted with"""
    available_files = list(filter(lambda x: os.path.exists(x), self.files))
    if len(available_files) == 0:
      return

    picked_file = prompts.ask_multiselect(available_files, "Select a file to delete", )

    for file_to_delete in picked_file:
      os.remove(file_to_delete)
      print(f"Deleted {file_to_delete}")


manager = _Manager()



import subprocess
from pyee.base import EventEmitter
from typing import Callable, Any
import re


class ProcessWithEvent():
  lines: list[str] = []
  current_line: str = ""

  def __init__(self, cmd: str,
               filter: Callable[[str], Any] = lambda x: x,
               progress_cb: Callable[[str, int], None] = lambda x, y: None):
    self.cmd = cmd
    self.filter = filter
    self.ee = EventEmitter()
    self.progress_cb = progress_cb

  def start(self):
    self.process = subprocess.Popen(self.cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, executable="/bin/zsh")
    self.ee.emit("start")

    while True:
      if not self.process.poll() is None or self.process.stdout is None:
        if len(self.current_line) > 0:
          self.handle_new_line()
        self.progress_cb(self.cmd, 100)
        break

      last = self.process.stdout.read(1)

      if last == b"\n" or last == b"\r":
        self.handle_new_line()
      else:
        self.current_line += last.decode("utf-8")

  def handle_new_line(self):
    self.lines.append(self.current_line)
    exp = r"(\d+\.\d+)(?=\s%)"
    match = re.search(exp, self.current_line)
    if match is not None:
      self.progress_cb(self.cmd, int(float(match.group(1))))

    elif match is None:
      # loader.loader.progress.log(self.current_line)
      pass

    self.current_line = ""



from setuptools import setup, find_packages


with open("requirements.txt", "r") as f:
  required_packages = f.read().splitlines()


setup(
    name="hardbrake",
    version="0.0.7",
    author="Tahsin",
    author_email="hello@tahsin.us",
    description="A wrapper around HandBrake CLI for encoding multiple files with ease.",
    packages=find_packages(),
    install_requires=required_packages,
    data_files=[("", ["requirements.txt"])],
    entry_points={
        'console_scripts': [
            'hardbrake = src.app:cli',
            'hb = src.app:cli'
        ]
    }
)



from rich.progress import Progress, TaskID


class Loader:
  progress = Progress()
  started = False
  tasks: dict[str, TaskID] = {}

  def add_task(self, task_id: str, total: int):
    self.tasks[task_id] = self.progress.add_task(task_id, total=total)

  def start(self):
    if self.started:
      raise Exception("Loader already started")

    self.started = True
    self.progress.start()

  def update(self, task_id: str, progress: int):
    self.check_if_started()
    self.progress.update(self.tasks[task_id], completed=progress)

  def check_if_started(self):
    if not self.started:
      raise Exception("Loader not started")

  def stop(self):
    self.check_if_started()
    self.progress.stop()
    self.progress.__exit__(None, None, None)
    self.started = False


loader = Loader()


def main():
  loader = Loader()

  loader.progress.update(loader.tasks["A"], completed=20)

  loader.progress.stop()


if __name__ == "__main__":
  main()



import src.logger as logger
import src.utils as utils
import src.manager as manager
import src.prompts as prompts
import src.file as file
from rich import traceback, print

traceback.install()


def main():
  try:
    try:
      utils.verify_installation("HandBrakeCLI")
      logger.success("HandBrakeCLI is installed")
    except Exception as e:
      logger.error(e.args[0])
      return

    happy = False
    original_files = file.select_files()

    if len(original_files) == 0:
      logger.error("No files selected")
      return

    for file_path in original_files:
      file.manager.add_file(file_path)

    while not happy:
      preset = utils.select_preset()

      cmds = [utils.get_encode_cmd(file_path, preset) for file_path in original_files]
      manager.manage(cmds)

      happy = prompts.ask_boolean("Are you happy with the results?")

    file.manager.delete_file()
    logger.success("Job done!")

  except KeyboardInterrupt:
    print("Exiting...")


def test():
  print("testing...")

  # file.manager.add_file('/Users/tahsin/Desktop/demvid/a.txt')
  # file.manager.add_file('/Users/tahsin/Desktop/demvid/b.txt')
  # file.manager.add_file('/Users/tahsin/Desktop/demvid/c.txt')
  # file.manager.add_file('/Users/tahsin/Desktop/demvid/foo.txt')

  # file.manager.delete_file()
  # pass
  # print('here')
